//-------------------------------------------------------------------------------------------------+
// GoalPoseSender.cpp
// Send goal pose datas to port
//-------------------------------------------------------------------------------------------------+

#include "GoalPoseSender.h"
#include <iostream>

using namespace std;


//-------------------------------------------------------------------------------------------------+
// Module specification
//-------------------------------------------------------------------------------------------------+
static const char* GoalPoseSender_spec[] =
{
    "implementation_id", "GoalPoseSender",
    "type_name",         "GoalPoseSender",
    "description",       "Send two dummy pose datas to port",
    "version",           "1.0.0",
    "vendor",            "aisl.tut.ac.jp",
    "category",          "dumm",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.goal_x", "0.0",
    "conf.default.goal_y", "0.0",
    "conf.default.goal_heading", "0.0",
    "conf.default.continuous_output", "0",
    "conf.default.deactivate_to_stop", "1",
    // Widget
    "conf.__widget__.goal_x", "text",
    "conf.__widget__.goal_y", "text",
    "conf.__widget__.goal_heading", "text",
    "conf.__widget__.continuous_output", "text",
    "conf.__widget__.deactivate_to_stop", "text",
    // Constraints
    ""
};


//-------------------------------------------------------------------------------------------------+
// RXgN^
//-------------------------------------------------------------------------------------------------+
GoalPoseSender::GoalPoseSender(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    m_goal_poseOut("goal_pose", m_goal_pose)
{
	cerr << "------------------------------------------------------------+\n"
		 << " GoalPoseSender Component (Ver. 8th December 2011)\n"
		 << "------------------------------------------------------------+\n\n";
}


//-------------------------------------------------------------------------------------------------+
// fXgN^
//-------------------------------------------------------------------------------------------------+
GoalPoseSender::~GoalPoseSender(){
}


//-------------------------------------------------------------------------------------------------+
// ֐
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GoalPoseSender::onInitialize(){
  
	// Set OutPort buffer
	addOutPort("goal_pose", m_goal_poseOut);

	// Bind variables and configuration variable
	bindParameter("goal_x", m_goal_x, "0.0");
	bindParameter("goal_y", m_goal_y, "0.0");
	bindParameter("goal_heading", m_goal_heading, "0.0");
	bindParameter("continuous_output", m_continuous_output, "0");

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// ANeBx[g̏
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GoalPoseSender::onActivated(RTC::UniqueId ec_id){

	m_goal_pose.data.length(1);
	m_goal_pose.data[0].position.x = m_goal_x;
	m_goal_pose.data[0].position.y = m_goal_y;
	m_goal_pose.data[0].heading    = m_goal_heading;

	m_goal_poseOut.write();

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// fBANeBx[g̏
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GoalPoseSender::onDeactivated(RTC::UniqueId ec_id){

	if( m_continuous_output && m_deactivate_to_stop ){

		m_goal_pose.data.length(0);
		m_goal_poseOut.write();
	}


	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// s֐
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GoalPoseSender::onExecute(RTC::UniqueId ec_id){

	if( !m_continuous_output ){

		ExecutionContext_var ec = this->get_context(0);
		ec->deactivate_component(this->getObjRef());

		return RTC::RTC_OK;
	}
	
	m_goal_poseOut.write();

	return RTC::RTC_OK;
}



//-------------------------------------------------------------------------------------------------+
extern "C"
{
 
  void GoalPoseSenderInit(RTC::Manager* manager)
  {
    coil::Properties profile(GoalPoseSender_spec);
    manager->registerFactory(profile,
                             RTC::Create<GoalPoseSender>,
                             RTC::Delete<GoalPoseSender>);
  }
};
